@@ -1,5 +1,5 @@
 /*
- * $Id: st-config-load.c,v 1.24 2004/01/30 15:04:36 jylefort Exp $
+ * $Id: st-config-load.c,v 1.35 2004/03/11 19:18:17 jylefort Exp $
  *
  * Copyright (c) 2002, 2003, 2004 Jean-Yves Lefort
  * All rights reserved.
@@ -33,10 +33,12 @@
 
 #include <glib.h>
 #include <glib-object.h>
+#include <string.h>
+#include <stdlib.h>
 #include "gettext.h"
 #include "sg-parser.h"
 #include "st-handler.h"
-#include "st-handler-field-api.h"
+#include "st-handler-field.h"
 #include "st-handlers.h"
 #include "st-parser.h"
 #include "st-action.h"
@@ -66,7 +68,10 @@ enum {
   STATEMENT_OPTIONS_STREAM_PROPERTIES_WINDOW_WIDTH,
   STATEMENT_OPTIONS_SELECTED_PREFERENCES_PAGE,
   STATEMENT_OPTIONS_PROXY_ENABLED,
-  STATEMENT_OPTIONS_PROXY_URL,
+  STATEMENT_OPTIONS_PROXY_TYPE,
+  STATEMENT_OPTIONS_PROXY_URL,			/* deprecated */
+  STATEMENT_OPTIONS_PROXY_SERVER,
+  STATEMENT_OPTIONS_PROXY_PORT,
   STATEMENT_OPTIONS_PROXY_AUTH_ENABLED,
   STATEMENT_OPTIONS_PROXY_AUTH_NAME,
   STATEMENT_OPTIONS_PROXY_AUTH_PASSWORD,
@@ -84,14 +89,14 @@ enum {
   STATEMENT_HANDLER_FIELDS_SORT_INDEX,
   STATEMENT_HANDLER_FIELDS_SORT_ORDER,
   STATEMENT_HANDLER_PANED_POSITION,
-  STATEMENT_HANDLER_SELECTED_CATEGORY,
-  STATEMENT_HANDLER_SELECTED_STREAM,		/* deprecated */
+  STATEMENT_HANDLER_SELECTED_CATEGORY,		/* obsolete */
+  STATEMENT_HANDLER_SELECTED_STREAM,		/* obsolete */
   STATEMENT_HANDLER_FIELDS_WIDTH,		/* deprecated */
   STATEMENT_HANDLER_FIELD,
   STATEMENT_HANDLER_FIELD_VISIBLE,
   STATEMENT_HANDLER_FIELD_WIDTH,
-  STATEMENT_HANDLER_EXPANDED_CATEGORIES,
-  STATEMENT_HANDLER_SELECTED_STREAMS
+  STATEMENT_HANDLER_EXPANDED_CATEGORIES,	/* obsolete */
+  STATEMENT_HANDLER_SELECTED_STREAMS		/* obsolete */
 };
 
 typedef struct
@@ -101,6 +106,7 @@ typedef struct
 
   char			*action;
   
+  GSList		*handlers;
   STHandler		*handler;
   GSList		*handler_field_iter;
 } STConfigLoadInfo;
@@ -197,9 +203,21 @@ static SGParserDefinition config_definitions[] = {
     "proxy_enabled",	FALSE,		G_TYPE_BOOLEAN
   },
   {
+    STATEMENT_OPTIONS,	STATEMENT_OPTIONS_PROXY_TYPE,
+    "proxy_type",	FALSE,		G_TYPE_INT
+  },
+  {				/* deprecated */
     STATEMENT_OPTIONS,	STATEMENT_OPTIONS_PROXY_URL,
     "proxy_url",	FALSE,		G_TYPE_STRING
   },
+  {
+    STATEMENT_OPTIONS,	STATEMENT_OPTIONS_PROXY_SERVER,
+    "proxy_server",	FALSE,		G_TYPE_STRING
+  },
+  {
+    STATEMENT_OPTIONS,	STATEMENT_OPTIONS_PROXY_PORT,
+    "proxy_port",	FALSE,		G_TYPE_INT
+  },
   {
     STATEMENT_OPTIONS,	STATEMENT_OPTIONS_PROXY_AUTH_ENABLED,
     "proxy_auth_enabled",	FALSE,	G_TYPE_BOOLEAN
@@ -260,11 +278,11 @@ static SGParserDefinition config_definitions[] = {
     STATEMENT_HANDLER,	STATEMENT_HANDLER_PANED_POSITION,
     "paned_position",		FALSE,	G_TYPE_INT
   },
-  {
+  {				/* obsolete */
     STATEMENT_HANDLER,	STATEMENT_HANDLER_SELECTED_CATEGORY,
     "selected_category",	FALSE,	G_TYPE_STRING
   },
-  {				/* deprecated */
+  {				/* obsolete */
     STATEMENT_HANDLER,	STATEMENT_HANDLER_SELECTED_STREAM,
     "selected_stream",	FALSE,		G_TYPE_STRING
   },
@@ -284,11 +302,11 @@ static SGParserDefinition config_definitions[] = {
     STATEMENT_HANDLER_FIELD,	STATEMENT_HANDLER_FIELD_WIDTH,
     "width",		FALSE,		G_TYPE_INT
   },
-  {
+  {				/* obsolete */
     STATEMENT_HANDLER,	STATEMENT_HANDLER_EXPANDED_CATEGORIES,
     "expanded_categories",	TRUE,	G_TYPE_NONE
   },
-  {
+  {				/* obsolete */
     STATEMENT_HANDLER,	STATEMENT_HANDLER_SELECTED_STREAMS,
     "selected_streams",	TRUE,		G_TYPE_NONE
   },
@@ -306,15 +324,15 @@ static void st_config_load_handler_field_end	     (STConfigLoadInfo *info);
 static void st_config_load_command		     (STConfigLoadInfo *info);
 static void st_config_load_program		     (STConfigLoadInfo *info);
 static void st_config_load_handler_field_width	     (STConfigLoadInfo *info);
-static void st_config_load_handler_expanded_category (STConfigLoadInfo *info);
-static void st_config_load_handler_selected_stream   (STConfigLoadInfo *info);
 
 /*** implementation **********************************************************/
 
 gboolean
 st_config_load (const char *filename, GError **err)
 { 
-  STConfigLoadInfo info = { 0, };
+  STConfigLoadInfo info = { NULL, };
+
+  g_return_val_if_fail(filename != NULL, FALSE);
 
   info.parser = sg_parser_new(filename, err);
   if (! info.parser)
@@ -401,11 +419,56 @@ st_config_load (const char *filename, GError **err)
 	      st_settings.proxy_enabled = g_value_get_boolean(&info.statement->value);
 	      break;
 	      
-	    case STATEMENT_OPTIONS_PROXY_URL:
-	      g_free(st_settings.proxy_url);
-	      st_settings.proxy_url = g_value_dup_string(&info.statement->value);
+	    case STATEMENT_OPTIONS_PROXY_TYPE:
+	      st_settings.proxy_type = g_value_get_int(&info.statement->value);
 	      break;
-	      
+
+	    case STATEMENT_OPTIONS_PROXY_URL:
+	      {
+		const char *proxy_url;
+		char *server = NULL;
+		int port = 0;
+		
+		proxy_url = g_value_get_string(&info.statement->value);
+		if (proxy_url)
+		  {
+		    const char *protocol;
+		    const char *portstr;
+
+		    /* skip protocol */
+
+		    protocol = strstr(proxy_url, "://");
+		    if (protocol)
+		      proxy_url = protocol + 3;
+
+		    /* extract port */
+
+		    portstr = strrchr(proxy_url, ':');
+		    if (portstr)
+		      {
+			server = g_strndup(proxy_url, portstr - proxy_url);
+			port = atoi(portstr + 1);
+		      }
+		    else
+		      server = g_strdup(proxy_url);
+		  }
+
+		g_free(st_settings.proxy_server);
+		st_settings.proxy_server = server;
+		st_settings.proxy_port = port;
+
+		break;
+	      }
+
+	    case STATEMENT_OPTIONS_PROXY_SERVER:
+	      g_free(st_settings.proxy_server);
+	      st_settings.proxy_server = g_value_dup_string(&info.statement->value);
+	      break;
+
+	    case STATEMENT_OPTIONS_PROXY_PORT:
+	      st_settings.proxy_port = g_value_get_int(&info.statement->value);
+	      break;
+
 	    case STATEMENT_OPTIONS_PROXY_AUTH_ENABLED:
 	      st_settings.proxy_auth_enabled = g_value_get_boolean(&info.statement->value);
 	      break;
@@ -425,6 +488,10 @@ st_config_load (const char *filename, GError **err)
 	    case STATEMENT_OPTIONS_GALEON_THEMES_ENABLED:
 	    case STATEMENT_OPTIONS_GALEON_THEMES_SYSTEM_DIR:
 	    case STATEMENT_OPTIONS_GALEON_THEMES_USER_DIR:
+	    case STATEMENT_HANDLER_SELECTED_CATEGORY:
+	    case STATEMENT_HANDLER_SELECTED_STREAM:
+	    case STATEMENT_HANDLER_EXPANDED_CATEGORIES:
+	    case STATEMENT_HANDLER_SELECTED_STREAMS:
 	      sg_parser_warn(info.parser, _("obsolete statement, ignored"));
 	      break;
 
@@ -455,32 +522,24 @@ st_config_load (const char *filename, GError **err)
 	      
 	    case STATEMENT_HANDLER_FIELDS_SORT_INDEX:
 	      if (info.handler)
-		info.handler->fields_sort_index = g_value_get_int(&info.statement->value);
+		{
+		  int sort_index = g_value_get_int(&info.statement->value);
+		  
+		  if (sort_index > st_handler_count_fields(info.handler, ST_HANDLER_FIELD_VISIBLE))
+		    sg_parser_warn(info.parser, _("index too big, ignored"));
+		  else
+		    st_handler_set_fields_sort_index(info.handler, sort_index);
+		}
 	      break;
 	      
 	    case STATEMENT_HANDLER_FIELDS_SORT_ORDER:
 	      if (info.handler)
-		info.handler->fields_sort_order = g_value_get_int(&info.statement->value);
+		st_handler_set_fields_sort_order(info.handler, g_value_get_int(&info.statement->value));
 	      break;
 	      
 	    case STATEMENT_HANDLER_PANED_POSITION:
 	      if (info.handler)
-		info.handler->paned_position = g_value_get_int(&info.statement->value);
-	      break;
-
-	    case STATEMENT_HANDLER_SELECTED_CATEGORY:
-	      if (info.handler)
-		{
-		  g_free(info.handler->selected_category_name);
-		  info.handler->selected_category_name = g_value_dup_string(&info.statement->value);
-		}
-	      break;
-	      
-	    case STATEMENT_HANDLER_SELECTED_STREAM:
-	      if (info.handler)
-		info.handler->selected_stream_names =
-		  g_slist_append(info.handler->selected_stream_names,
-				 g_value_dup_string(&info.statement->value));
+		st_handler_set_paned_position(info.handler, g_value_get_int(&info.statement->value));
 	      break;
 
 	    case STATEMENT_HANDLER_FIELD:
@@ -491,7 +550,7 @@ st_config_load (const char *filename, GError **err)
 	      if (info.handler_field_iter)
 		{
 		  STHandlerField *field = info.handler_field_iter->data;
-		  field->user_visible = g_value_get_boolean(&info.statement->value);
+		  st_handler_field_set_user_visible(field, g_value_get_boolean(&info.statement->value));
 		}
 	      break;
 
@@ -499,7 +558,7 @@ st_config_load (const char *filename, GError **err)
 	      if (info.handler_field_iter)
 		{
 		  STHandlerField *field = info.handler_field_iter->data;
-		  field->width = g_value_get_int(&info.statement->value);
+		  st_handler_field_set_width(field, g_value_get_int(&info.statement->value));
 		}
 	      break;
 	    }
@@ -513,13 +572,10 @@ st_config_load (const char *filename, GError **err)
 	      break;
 	      
 	    case STATEMENT_HANDLER_EXPANDED_CATEGORIES:
-	      st_config_load_handler_expanded_category(&info);
+	    case STATEMENT_HANDLER_SELECTED_STREAMS:
+	      /* obsolete */
 	      break;
 
-	    case STATEMENT_HANDLER_SELECTED_STREAMS:
-	      st_config_load_handler_selected_stream(&info);
-	      break;
-	      
 	    default:
 	      sg_parser_warn(info.parser, _("unexpected value"));
 	    }
@@ -530,7 +586,8 @@ st_config_load (const char *filename, GError **err)
     }
   
   sg_parser_free(info.parser);
-
+  st_handlers_reorder(info.handlers);
+  
   return TRUE;
 }
 
@@ -543,7 +600,10 @@ st_config_load_handler_begin (STConfigLoadInfo *info)
   info->handler = st_handlers_find_by_name(name);
   
   if (info->handler)
-    info->handler_field_iter = info->handler->fields;
+    {
+      info->handler_field_iter = st_handler_get_fields(info->handler);
+      info->handlers = g_slist_append(info->handlers, info->handler);
+    }
   else
     sg_parser_warn(info->parser, _("%s: no such handler"), name);
 }
@@ -565,7 +625,7 @@ st_config_load_handler_field_begin (STConfigLoadInfo *info)
     {
       STHandlerField *field = info->handler_field_iter->data;
       
-      if (field->visible)
+      if (ST_HANDLER_FIELD_IS_VISIBLE(field))
 	break;
       
       info->handler_field_iter = info->handler_field_iter->next;
@@ -644,48 +704,14 @@ st_config_load_handler_field_width (STConfigLoadInfo *info)
 
       info->handler_field_iter = info->handler_field_iter->next;
 
-      if (field->visible)
+      if (ST_HANDLER_FIELD_IS_VISIBLE(field))
 	break;
       else
 	field = NULL;
     }
   
   if (field)
-    field->width = g_value_get_int(&info->statement->value);
+    st_handler_field_set_width(field, g_value_get_int(&info->statement->value));
   else
     sg_parser_warn(info->parser, _("too many fields"));
 }
-
-static void
-st_config_load_handler_expanded_category (STConfigLoadInfo *info)
-{
-  if (! info->handler)
-    return;
-
-  if (! G_VALUE_HOLDS_STRING(&info->statement->value))
-    {
-      sg_parser_warn(info->parser, _("expected string value"));
-      return;
-    }
-  
-  info->handler->expanded_categories
-    = g_slist_append(info->handler->expanded_categories,
-		     g_value_dup_string(&info->statement->value));
-}
-
-static void
-st_config_load_handler_selected_stream (STConfigLoadInfo *info)
-{
-  if (! info->handler)
-    return;
-
-  if (! G_VALUE_HOLDS_STRING(&info->statement->value))
-    {
-      sg_parser_warn(info->parser, _("expected string value"));
-      return;
-    }
-  
-  info->handler->selected_stream_names
-    = g_slist_append(info->handler->selected_stream_names,
-		     g_value_dup_string(&info->statement->value));
-}
